#!/usr/bin/python -u

import getopt
import binascii
import __builtin__

try:
	import MAPI
	from MAPI.Util import *
	from MAPI.Time import *
	from MAPI.Struct import *
except ImportError, e:
	print "Not all modules can be loaded. The following modules are required:"
	print "- MAPI (Zarafa)"
	print ""
	print e
	sys.exit(1)

def print_help():
	print "Usage: %s -u [username of mailbox]" % sys.argv[0]
	print ""
	print "Delete all rules for specified user"
	print ""
	print "Required arguments:"
	print "   -u, --user          user to set forwarding rule"
	print "   -h, --host          Host to connect with. Default: file:///var/run/zarafa"
	print "   -s, --sslkey-file   SSL key file to authenticate as admin. Default: /etc/zarafa/ssl/client.pem"
	print "   -p, --sslkey-pass   Password for the SSL key file."
	print ""
	print "optional arguments:"
	print "   --help              Show this help message and exit."

def UnixTimeToCOleDateTime(unixtime):
    days = (float(unixtime)/(60*60*24))+25569
    cOleDateTime = struct.pack("d", days)
    return cOleDateTime

def main(argv = None):
	if argv is None:
		argv = sys.argv

	try:
		opts, args = getopt.gnu_getopt(argv, 'u:h:s:p:', ['user=', 'host=', 'sslkey-file=', 'sslkey-pass='])
	except getopt.GetoptError, err:
		# print help information and exit:
		print str(err)
		print ""
		print_help()
		return 1

	host = 'file:///var/run/zarafa'
	sslkey_file = '/etc/zarafa/ssl/client.pem'
	sslkey_pass = None
	username = None

	for o, a in opts:
		if o in ('-h', '--host'):
		    host = a
		elif o in ('-s', '--sslkey-file'):
		    sslkey_file = a
		elif o in ('-p', '--sslkey-pass'):
		    sslkey_pass = a
		elif o in ('-u', '--user'):
		    username = a
		elif o == '--help':
		    print_help()
		    return 0
		else:
		    assert False, ("unhandled option '%s'" % o)

	if not username:
		print "No username specified."
		print ""
		print_help()
		sys.exit(1)

	try:
		session = OpenECSession(username, '', host, sslkey_file = sslkey_file, sslkey_pass = sslkey_pass)
		store = GetDefaultStore(session)
		inboxeid = store.GetReceiveFolder('IPM', 0)[0]
		inbox = store.OpenEntry(inboxeid, None, MAPI_BEST_ACCESS)
		inbox.DeleteProps([PR_RULES_TABLE])

	except MAPIError, err:
		if err.hr == MAPI_E_LOGON_FAILED:
		    print "Failed to logon. Make sure your credentials are correct."
		elif err.hr == MAPI_E_NETWORK_ERROR:
		    print "Unable to connect to server. Make sure you specified the correct server."
		else:
		    print "Unexpected error occurred. hr=0x%08x" % err.hr
		sys.exit(1)

if __name__ == '__main__':
    #locale.setlocale(locale.LC_ALL, '')
    sys.exit(main())
