#!/bin/bash

#####################################################################
# $Id: zarafa-safe-shutdown.sh,v 1.4 2014/07/16 10:03:40 rusjako Exp $
#####################################################################
# Written and maintained by:
#    Stefan Jakobs <stefan.jakobs at tik.uni-stuttgart.de>
#
# Please send all comments, suggestions, bug reports,
#    etc, to mail-support at tik.uni-stuttgart.de
#####################################################################
# Copyright (c) 2013 Stefan Jakobs
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
#
#####################################################################

# Beschreibung


#### Set Environment ####
PATH="$PATH:/usr/bin/:/usr/sbin/:/bin:/sbin"

### Set Variables

mysql_user='zarafa'
mysql_client='/usr/bin/mysql'
mysql_socket='/var/run/mysqld/mysqld.sock'
debug=${debug:-0}
status=0

#### Functions ####

## Flush zarafa database tables
flush_zarafa_tables() {
   local status=0
   if [ -e $mysql_socket ]; then
      echo 'show tables;' | $mysql_client -N $mysql_user | while read table; do
         if [ $debug -eq 1 ]; then 
            echo "${HOSTNAME}: $(date "+%d.%m.%Y %k:%M:%S flushing $table")"
         fi
         echo "flush table $table;" | $mysql_client -N $mysql_user
         if [ "$?" -eq 0 ]; then
            if [ $debug -eq 1 ]; then
               echo "${HOSTNAME}: $(date "+%d.%m.%Y %k:%M:%S flushing $table OK")"
            fi
         else
            echo "${HOSTNAME}: $(date "+%d.%m.%Y %k:%M:%S flushing $table FAILED")"
            status=1
         fi
      done
  else
     if [ $debug -eq 1 ]; then
        echo "${HOSTNAME}: info: skip flushing tables; mysql socket not available!"
     fi
  fi
  return $status
}

#### MAIN ####

## check for a running mysqldump
if ps -fC mysqldump >/dev/null ; then
   echo "${HOSTNAME}: error: mysqldump is running. Shutdown canceled!"
   exit 1
fi
## disable puppet
if ps -fC puppet >/dev/null; then 
  puppet_status="$(service puppet stop)"
  if [ "$debug" -eq 1 ]; then
     echo "${HOSTNAME}: ${puppet_status}"
  fi
fi
if ! flush_zarafa_tables ; then
   echo "${HOSTNAME}: error: flushing mysql tables failed!"
   exit 1
elif ! service zarafa-server stop > /dev/null; then
   echo "${HOSTNAME}: error: failed to stop zarafa-server!"
   exit 1
elif ! flush_zarafa_tables ; then
   echo "${HOSTNAME}: error: flushing mysql tables failed!"
   exit 1
elif ! sync ; then
   echo "${HOSTNAME}: error: failed to sync disks!"
   exit 1
else
   if [ "$debug" -eq 1 ]; then
      echo "${HOSTNAME}: flush tables and disc sync finished"
      echo "${HOSTNAME}: check state: $(service zarafa-server status)"
   fi
fi
if ! service mysql stop > /dev/null; then
   echo "${HOSTNAME}: warning: failed to stop mysql daemon! continue anyway ..."
elif [ "$debug" -eq 1 ]; then
   echo "${HOSTNAME}: check state: $(service mysql status)"
fi

## enable sysrq-trigger
echo 1 > /proc/sys/kernel/sysrq
## sync disk via sysrq
echo s > /proc/sysrq-trigger
## disable sysrq-trigger
echo 0 > /proc/sys/kernel/sysrq

if ! sync ; then
   echo "${HOSTNAME}: error: failed to sync disks (2nd time)!"
   exit 1
elif ! [[ "$(service mysql status)" =~ 'stop' ]] ; then
   if ! service mysql stop > /dev/null; then 
      echo "${HOSTNAME}: error: failed to stop mysql daemon!"
      exit 1
   fi
fi

## kill zarafa-server if it is still running
counter=5
while ps -C zarafa-server > /dev/null; do
   if [ "$counter" -eq 0 ]; then
      if killall -KILL zarafa-server ; then
         echo "${HOSTNAME}: warning: zarafa-server killed"
      fi
   else
      (( counter = $counter - 1 ))
      sleep 60
      killall -TERM zarafa-server
      echo "${HOSTNAME}: warning: send SIGTERM to zarafa-server"
   fi
done

#### End of MAIN ####
